;; FIR filter with coefficient generation   y(k) = Sum{x(k-n)*h(n)} n = 0..(N-1)
;; Syntax:  fir2  name,N,flo,fhi,abs|rel    h(n) = w(n)*{Sin(2*pi*fhi*n)-sin(2*pi*flo*n)}/(pi*n)
;; where: N=number of taps; flo, fhi = bandpass edge freqs; abs= freqs in Hz, rel= freqs in Fs/2 
;; Input: name_in	Output: name  
;; Optional output: name_delay (delayed input by N/2Fs) 
;; for group delay compensation


fir2	 macro	name,points,flo,fhi,unit

 if	 ("unit"=='Hz')||("unit"=='abs') 
name\_fu	set	2*flo/actual_fs
name\_fd	set	2*fhi/actual_fs
 else  
  if	 ("unit"=='kHz')
name\_fu	set	2000.*flo/actual_fs
name\_fd	set	2000.*fhi/actual_fs
  else 
   if	("unit"=='MHz')
name\_fu	set	2E6*flo/actual_fs
name\_fd	set	2E6*fhi/actual_fs
   else 
name\_fu	set	flo
name\_fd	set	fhi
   endif
  endif
 endif 


;; Coefficient table 
	section	far	global
	org	y:
name\_coeffs

name\_inc	set	2.0*pi/@cvf(points-1)
name\_w0	set	0.35875
name\_w1	set	0.48829
name\_w2	set	0.14128
name\_w3	set	0.01168

name\_angle	set	0.0
name\_num	set	(1.-@cvf(points))/2.

	dup	points
name\_w	set	name\_w0-name\_w1*@cos(name\_angle)+name\_w2*@cos(2.0*name\_angle)-name\_w3*@cos(3.0*name\_angle)
name\_angle	set	name\_angle+name\_inc

	if	name\_num==0.
	dc	name\_w*(name\_fd-name\_fu)
	else
	dc	name\_w*(@Sin(pi*name\_fd*name\_num)-@Sin(pi*name\_fu*name\_num))/(pi*name\_num)
	endif
name\_num	set	name\_num+1.0

	endm
	endsec



	bufm	name\_states,x,points

	org	p:
name\_exe
	ptr	name\_s,0,name\_states,points-1,(points+1)/2	
	ptr	name\_c,4,name\_coeffs,-1,	
	ldx0	name\_in

	if	@def(name\_delay)
	move		x:(r0+n0),a
	sta		name\_delay
	endif

	clr	a	x0,x:(r0)-	y:(r4)+,y0
	do	m0,name\_1
	mac	x0,y0,a	x:(r0)-,x0	y:(r4)+,y0	
name\_1
	if	@msp(name)==3
	mac	x0,y0,a		(r0)+
	else
	macr	x0,y0,a		(r0)+
	endif
	sta		name
	ptru		name\_s
	move		#-1,m0
	move		m0,m4
; Used registers for sample execution
	u	x
	u	y
	u	a
	u	r0
	u	n0
	u	r4


	endm		

;; DSP56300 ASM Code
;; Written by Jean-Marie ORY
;; Free software restricted to academic use


