;; Hilbert transformer (FIR filter introducing 90 phase shift) H(n) = [1 - Cos(n*pi)] / n*pi
;; Syntax:	hilbert	name,points
;; points = FIR length (more points = more precision at lower frequencies)
;; Input = name_in (real) Output = name (complex) or name_re, name_im (real)
;; Note: Odd value of "points" produces coeff table reduced by 2, thus ~ 2x faster code. 

hilbert	macro	name,points

 if	!@def(hilb_coef\?points)
	section	data	global

  if	points%2==0
hilb_n	set		@cvf(1-points)/2.
	org	y:
	buffer		m,points
hilb_coef\?points
		dup		points
		dc		(1.-@cos(hilb_n*pi))/(hilb_n*pi)
hilb_n	set		hilb_n+1.0
		endm
	endbuf
  else


hilb_n	set		@cvf((1-points)/2)
	org	y:

hilb_coef\?points
	dup	points
hilb_num 	set 		1.-@cos(hilb_n*pi)
   if	hilb_num!=0
		dc		2./(hilb_n*pi)
   endif
hilb_n	set		hilb_n+1.0
	endm
  endif
	endsec
 endif

	section	vars	global
	org	x:
	buffer	m,points
name\_dly
	ds		points
	endbuf
	endsec


	org	p:
name\_exe
  if	points%2==0
		ptr		name\_s,0,name\_dly,points-1,-points/2
		ptr		name\_c,4,hilb_coef\?points,-1,0

   if		!(rgin)
		lda		name\_in
   endif

	move			a,x:(r0)
	move			x:(r0+n0),x1
	clr	a		x:(r0)-,x0	y:(r4)+,y0
	do			#points-1,name\_ex1
	mac			x0,y0,a	x:(r0)-,x0	y:(r4)+,y0
name\_ex1
	macr			x0,y0,a		(r0)+
	sta			name\_im
	move			x:(r0+n0),a
	add			x1,a
	asr			a
	sta			name\_re
	ptru			name\_s

 else
	ptr			name\_s,0,name\_dly,points-1,-2
	ptr			name\_c,4,hilb_coef\?points,-1

	lda			name\_in
  if	((points-1)/2)%2==0
	move			a,x:(r0)-
  else
	move			a,x:(r0)
  endif
	clr	a		x:(r0)+n0,x0	y:(r4)+,y0

  if	((points-1)/2)%2==0
	do			#(((points-1)/2)-1),name\_ex1
  else
	do			#((points-1)/2),name\_ex1
  endif
	mac			x0,y0,a	x:(r0)+n0,x0	y:(r4)+,y0
name\_ex1
   if	((points-1)/2)%2==0
	macr			x0,y0,a		(r0)+
   else
	macr			x0,y0,a		(r0)-n0
   endif
	ptru			name\_s
	move			#(points-1)/2,n0
	sta			name\_im
	move			x:(r0+n0),a
	sta			name\_re

 endif

	u	a
	u	x
	u	y
	u	r0
	u	m0
	u	n0
	u	r4
	u	m4

	endm





;; DSP56300 ASM Code
;; Written by Jean-Marie ORY
;; Free software restricted to academic use


