;; Install acquisition and PWM with interrupt, execute file at each interrupt
;; Syntax:	irqpwm	name,timer,tpwm,tmin,tmax,'file_name'   where timer is the H/W timer used (0,1,or 2) If 0, ADA is performed;
;; tpwm, tmin, tmax = pwm period, min and max ratio; 'file_name' = program to execute at each interrupt
;; Inputs: name_in (da1, da2 if timer = 0) Outputs: ad1,ad2 (if timer = 0)


irqpwm	macro	name,timer,tpwm,tmin,tmax,filename


;	if	@cnt()==6
Npwm	set	@cvi(tpwm*fdsp/2.)
Npwmin	set	@cvi(tmin*fdsp/2.)
Npwmax	set	@cvi(tmax*fdsp/2.)


	if	timer==0
; Timer0 defs
tcsr 	set	tcsr0
tlr 	set	tlr0
tcpr	set	tcpr0
tcr	set	tcr0
tcfirq	set	$24
tofirq	set	$26
	endif

	if	timer==1
; Timer1 defs
tcsr 	set	tcsr1
tlr 	set	tlr1
tcpr	set	tcpr1
tcr	set	tcr1
tcfirq	set	$28
tofirq	set	$2a
	endif

	if	timer==2
; Timer2 defs
tcsr 	set	tcsr2
tlr 	set	tlr2
tcpr	set	tcpr2
tcr	set	tcr2
tcfirq	set	$2c
tofirq	set	$2e
	endif

actual_tpwm	set	(2.*@cvf(@cvi(fdsp*tpwm/2.)))/fdsp	
actual_fs	set	1./actual_tpwm



	var	name\_nwtcpr,-Npwmin


 if	!@def(pwm_isr)

	setirq	tov\timer,3,pwm_isr

  begint	pwm_isr

	if	@msp(name\_nwtcpr)==1
	movep	x:name\_nwtcpr,x:tcpr
	else
	movep	y:name\_nwtcpr,x:tcpr
	endif
   if		timer==0
;	var	ad1,0
;	var	ad2,0
;	var	da1,0
;	var	da2,0

  if	@def(ad1)
	if	@msp(ad1)==2
	movep	y:$fffffe,y:ad1
	else
	movep	y:$fffffe,x:ad1
	endif
  endif

  if	@def(ad2)
	if	@msp(ad2)==2
	movep	y:$ffffff,y:ad2
	else
	movep	y:$ffffff,x:ad2
	endif
  endif

  if	@def(da1)
	if	@msp(da1)==2
	movep	y:da1,y:$fffffe
	else
	movep	x:da1,y:$fffffe
	endif
  endif

  if	@def(da2)
	if	@msp(da2)==2
	movep	y:da2,y:$ffffff
	else
	movep	x:da2,y:$ffffff
	endif
  endif
   endif

	if	"filename"!=''
	include	"filename"
	else
	flagset	name
	endif

	lda	name\_in
	asr	a
	add	#0.5,a
	move	#>Npwmax-Npwmin,x1
	move	a,x0
	mpyr	-x0,x1,a
	add	#>-Npwmin,a
	sta	name\_nwtcpr
	
	u	a
	u	x
  endint	pwm_isr



	section	inits
	org	p:
init_pwm\timer		

; periode = 2*tcpr/fdsp
; x x tcf tof x x x x pce x do di dir x trm inv tc3..tc0 x tcie toie te
; 0 0  1   1  0 0 0 0  0  0 0  0  0   0  1   0   0111    0  0    1    1
;     3      .   0    .    0     .      A      .   7   .      3

	movep	#-Npwm,x:tlr
	movep	#0,x:tcpr
	movep	#$300B71,x:tcsr
	brclr	#20,x:tcsr,*	; wait for TOF
	move	#>-Npwmin,x0
	move	x0,x:tcpr
	stx0	name\_nwtcpr
	movep	#$300B73,x:tcsr
	endsec
 endif

	endm

;; DSP56300 ASM Code
;; Written by Jean-Marie ORY
;; Free software restricted to academic use


