;; 2D function approximation by 2D-table read / interpolate
;; Syntax: tblr2d	name,table,xsize,xsu,ysu
;; 'table' is a text file 'table.asm' of xsize * ysize values (records in the form:'  DC nn,nn,..' ) 
;; or 'table' is the address of a table defined in Y: ; xsize is the matrix width;     
;; xsu= 's' or 'u'  defines wether name_inx is signed or unsigned, idem for ysu
;; JMO Rev. 7/11/05
;; tblr2d.bmp

;; 2D Linear interpolation is performed between points  
;; 

;; Signed input: xsu = 's' or ysu = 's'
;; Values are given for equally spaced inputs  between -1 and +1 :   x(k) = -1.+2k/(n-1) k=0..(n-1) 
;; The first table value is the nl output for in = -1.0
;; The last table value is the nl output for in = +1.0 (never reached)

;; Unsigned input: xsu = 'u' or ysu = 'u'
;; Values are given for equally spaced inputs between 0 and +1 :  x(k) = k/(n-1) k=0..(n-1) 
;; The first table value is the nl output for in = 0.0
;; The last table value is the nl output for in = +1.0 (never reached)


tblr2d	macro	name,table,xsize,xsu,ysu

	if	!@def(table)
	section	data	global
		org	y:
table\_sect	set	3
table	
		include	"table"++'.asm'
table\_end	equ	*

		if	((table\_end-table)%(xsize))!=0
		warn	'Table length not multiple of xsize !'
		endif
table\_xsegmts	equ	xsize-1
table\_ysegmts	equ	((table\_end-table)/(xsize))-1

		if	table\_xsegmts<1
		fail	'Xsize (columns) should be > 1 !'
		endif
		if	table\_ysegmts<1
		fail	'Ysize (rows) should be > 1 !'
		endif
	endsec
	endif

		org	p:
		cc_rst
name\_exe
	if	!rgin
	lda	name\_inx
	ldb	name\_iny
	endif


	move	#>table\_xsegmts,x0
	move	#>table\_ysegmts,y0
	move	#table,r0



	if	!("xsu"=='u')
	asr	a	#0.5,y1
	add	y1,a			; b = (y + 1.0) / 2
	endif

	if	!("ysu"=='u')
	asr	b	#0.5,y1
	add	y1,b			; b = (y + 1.0) / 2
	endif



	jsr	tb2dadr
	cc_add	18	
	jsr	tbrd2d
	cc_add	13

	sta	name
	cc_ac	1



	dbgw	TBLR2D
	dbg	name\_inx
	dbg	name\_iny
	dbg	name




	if	!@def(tb2dadr)
	section	routines
	global	tb2dadr

;; Calculate address of Z(i,j), and fractions to i+1 and j+1

;; input:
;; a = unsigned Xin	B = unsigned Yin	R0--> Begin of table
;; x0= xsegmts		y0= ysegmts

;; output:
;; R0--> Z(ij)	N0 = Xsize
;; X1= fraction X	Y1= fraction Y

tb2dadr	move	a,y1
	mpy	x0,y1,a	b,y1		; a = xsegmts*xin
	mpy	y0,y1,b			; b = ysegmts*yin
	lsl	a			; 
	lsl	b	b1,y1		; y1 = j
	asr	a			; a1 = i  	a0 = fx	
	asr	b	a0,x1		; b1 = j  	b0 = fy		x1 = fx
	add	y1,a	r0,y0		; a1 = i+j	y0 = Table
	add	y0,a			; a1 = i+j+table
	asr	#23,a,a			; a0 = 2*(i+j+table)
	mac	x0,y1,a	b0,y1		; a0 = 2*(i+j+table+j*xsegmts)	y1 = fy
	asr	a	#>1,b		; a0 = table + j*xsize + i	b1 = 1
	add	x0,b	a0,r0		; b1= 1+xsegmts			R0 -> Z(i,j)
	move	b1,n0			; N0 = xsize 		
	rts
	endsec
	endif



	if	!@def(tbrd2d)
	section	routines
	global	tbrd2d
	org	p:



;; R0 --> z(ij) 	N0 = Xsize
;; avec z(i,j) = Z00
;; z = (1-fx)(1-fy)Z00 + fx(1-fy)Z01 + fx*fy*Z11 + (1-fx)*fy*Z10


tbrd2d	move	y:(r0)+,a		; a = z00
	mpy	x1,y1,b		a,x0	; b = fx*fy x0=Z00

	mac	-x0,x1,a	b,y0		; a = z00 -fx*z00	y0= fx*fy
	mac	-x0,y1,a			; a = z00 -fx*z00 -fy*z00
	mac	x0,y0,a    y:(r0)+n0,x0	; a = z00 -fx*z00 -fy*z00 +fx*fy*z00   x0 = z01

	mac	x0,x1,a			; a+= fx*z01
	mac	-x0,y0,a   	y:(r0)-,x0	; a+= -fx*fy*z01	x0 = z11

	mac	x0,y0,a	y:(r0)-n0,x0	; a+= fx*fy*z11  x0 = z10

	mac	x0,y1,a			; a+= fy*z10
	macr	-x0,y0,a			; a+= -fx*fy*z01
	rts

	endsec
	endif

	u	a
	u	b
	u	x
	u	y
	u	r0
	u	n0
	u	m0
	varin
   endm				

;; DSP56300 ASM Code
;; Written by Jean-Marie ORY
;; Free software restricted to academic use


