;; Function approximation by table read / interpolate
;; Syntax: tblrd	name,table,su
;; table is either an address or a filename (without extension .asm)
;; if table is an address, define table_segmts= size-1
;; Signed / unsigned input according to su = 's' or su = 'u'
;; JMO Rev. 7/11/05
;; tblrd.bmp

;; Either:
;; table is the address of a previously defined table
;; and table_segmts is the number of linear segments (table points - 1)
;; Or:
;; 'table' is the pathname of an ascii file 
;; containing n values between [-1 .. +1[  
;; in the format :
;; 	dc	0.123,0.234, ...
;;	dc	0.456,0.567, ...
;;	...

;; Linear interpolation is performed between points  
;; 
;; Signed input: tblrd  name,table,s
;; Values are given for equally spaced inputs  x(k) = -1.+2k/(n-1) k=0..(n-1) 
;; The first table value is the nl output for in = -1.0
;; The last table value is the nl output for in = +1.0 (never reached)

;; Unsigned input: tblrd  name,table,u
;; Values are given for equally spaced inputs x(k) = -1. +2k/(n-1) (s)
;;                                        or  x(k) = k/(n-1)       (u)
;;                                                          k=0..(n-1) 
;; The first table value is the nl output for in = -1. (s)  or 0.0 (u)
;; The last table value is the nl output for in = +1.0 (never reached)


tblrd	macro	name,table,su
	section	far	global
	if	!@def(table)
	org	y:
table\_sect set	3
table	
	include	"table"++'.asm'

table\_size	set	*-table
	endif
table\_segmts	set	table\_size-1
	endsec

		org	p:

exe_\name
	if	!rgin
	lda	name\_in
	endif
	move	#table,r0
	move	#-1,m0
	move	#>table\_size-1,x0

	if	!("su"=='u')
name\_sgn equ	1
	asr	a	#0.5,y1
	add	y1,a			;; a = (x + 1.0) / 2
	endif


	move	a,y0			;; y0= unsigned in
	mpy	x0,y0,a		;; a = u_in*segmts
	lsl	a
	asr	a			;; a1 = int = k ; a0 = pos fract
	move	a1,n0			;; n0 = int(")	
	move	a0,x1			;; x1 = fract(")
	move	(r0)+n0			;; r0 = absc(yk)

	if	@msp(table)==1
	move	x:(r0)+,a		;; a  = Yk
	move	x:(r0),y1		;; y1 = Yk+1
	endif

	if	@msp(table)==2
	move	y:(r0)+,a		;; a  = Yk
	move	y:(r0),y1		;; y1 = Yk+1
	endif

	if	@msp(table)==4
	move	p:(r0)+,a		;; a  = Yk
	move	p:(r0),y1		;; y1 = Yk+1

	endif

	mac	x1,y1,a		a,y0	;; a  = yk +fract*yk+1    y0=yk
	macr	-x1,y0,a		;; a  = yk +fract*(yk+1 -yk)		


	sta	name

	u	a
	u	x
	u	y
	u	r0
	u	n0
	u	m0


   endm				

;; DSP56300 ASM Code
;; Written by Jean-Marie ORY
;; Free software restricted to academic use


