;; Discrete Cosine Transform
;; 

dct	macro	name

 if !@def(fftbuf)	
	section	longfar		global
		org		L:
n		set		name\_in_size

		buffer	r,n
fftbuf	
		ds		n
		endbuf
fftbuf_size	equ	n
fftbuf_lin	equ	n
fftbuf_col	equ	1
fftbuf_field	equ	3
	endsec
 endif
   

;        include 'fftr2aa'

	section		longfar global
		org		l:
delta	set		2.0*pi/@cvf(name\_size)
angle	set		0.0
		buffer	r,name\_size/2
fftcoef
		dup		name\_size/2
		dc		@lng(@frc(-@cos(angle)),@frc(-@sin(angle)))
angle	set		angle+delta
		endm
		endbuf



;       Generate exponential table after twiddle factor table.
;       This generates the scale factor of EXP(-j*pi*k/(2*N)).
;       The real part is in Y memory and the imaginary part is
;       in X memory.  Note that the negative of the part in Y memory
;       is actually stored so that a -1 (exactly) can be represented
;       with the fractional arithmetic.  This negative sign is
;       compensated when the actual rotation is performed.
;

delta	set		pi/@cvf(2*name\_size)
angle	set		0
exptbl
		dup		name\_size
		dc		@lng(@frc(-@sin(angle)),@frc(-@cos(angle)))
angle	set		angle+delta
		endm
		endsec

;

;       Do inital sort. The input data is in Y, the sorted data goes to X.
;       This forms a new sequence in X memory from the sequence in Y memory
;       according to the definition:
;               x(k)=y(2k)
;               x(N-1-K)=y(2k+1)      k=0,1,...,N/2-1
;
	org		p:
 if @def(name\_busy_b)
	flagset	name\_busy
 endif
	move		#-1,m0			;set linear
	move		#name\_in,r0		;point to data
	move		#fftbuf,r4
	move		#fftbuf+n-1,r5
	move		#n/2,n0			
;	move		r0,r4			;copy data pointer
	move		m0,m4			;ditto
	move		m0,m5
	move		#0,x0
	do		n0,name\_1
 if	@msp(name\_in)==2
	move		y:(r0)+,x1
	move		x,l:(r4)+
	move		y:(r0)+,x1
	move		x,l:(r5)-
name\_1
 else
	move		x:(r0)+,x1
	move		x,l:(r4)+
	move		x:(r0)+,x1
	move		x,l:(r5)-
name\_1
 endif

;       Do an FFT on the new sequence
;
;//        fftr2aa  name\_size,name,name\_coeff        ;do FFT

;; FFT (adapted from Motorola DSP56300 Library FFTAS.ASM )
         move    #n/2,n0      ;initialize butterflies per group
         move    #1,n2             		;initialize groups per pass
         move    #n/4,n6      ;initialize C pointer offset
         move    #-1,m0            		;initialize A and B address modifiers
         move    m0,m1             ;for linear addressing
         move    m0,m4
         move    m0,m5
         move    #0,m6             ;initialize C address modifier for
                                   ;reverse carry (bit-reversed) addressing
         andi    #$7F,ccr          ;clear scaling bit
         ori     #$4,mr            ;set scaling down mode

;
; Perform all FFT passes with triple nested DO loop
;
         do      #@cvi(@log(n)/@log(2)+0.5),_end_pass
         move    #fftbuf,r0		;initialize A input pointer
         move    r0,r4				;initialize A output pointer
         move    n0,n1				;initialize pointer offsets
         move    n0,n4
         lua     (r0)+n0,r1			;initialize B input pointer
         move    #fftcoef,r6		;initialize C input pointer
         move    n0,n5
         lua     (r1)-,r5				;initialize B output pointer

         do      n2,_end_grp
         move    x:(r1),x1  y:(r6),y0	;lookup -sine and -cosine values
         move    x:(r5),a   y:(r0),b      ;preload data
         asl     a          x:(r6)+n6,x0  ;since scaling mode on a=2*a,update C pointer

         do      n0,_end_bfy
         mac     x1,y0,b    y:(r1)+,y1    ;Radix 2 DIT ;butterfly kernel
         macr    -x0,y1,b   a,x:(r5)+  y:(r0),a
         subl    b,a        
	   	move    x:(r0),b   b,y:(r4)
         mac     -x1,x0,b   x:(r0)+,a  a,y:(r5)
         macr    -y1,y0,b   x:(r1),x1
         subl    b,a        
	   	move    b,x:(r4)+  y:(r0),b
_end_bfy
         move    a,x:(r5)+n5    y:(r1)+n1,y1   ;update A and B pointers
         move    x:(r0)+n0,x1   y:(r4)+n4,y1
_end_grp
         move    n0,b1
         lsr     b   n2,a1			;divide butterflies per group by two
	   nop
         lsl     a   b1,n0			;multiply groups per pass by two
	   nop
         move    a1,n2
_end_pass
         andi    #$7f,ccr			;clear scaling bit
         andi    #$f3,mr			;no scaling



;

;       Multiply the output by a complex exponential and take the
;       real part.  dct=re[ (a+jb)*(c+jd)] = a*c-b*d where a+jb is the
;       output of the FFT and c+jd is the rotation factor. Note that
;       the data is still in bit reversed order but the rotation
;       table is in sequential order.  Thus, the data is accessed with
;       bit reversed addressing but the rotation table is addressed
;       with linear addressing.

        move    #exptbl,r0              ;point to exponential table
        move    #-1,m0	                ;linear addressing
		move		#n-1,n0
        move    #fftbuf,r4                ;point to output of fft
        move    #0,m4                   ;bit reversed addressing
        move    #n/2,n4
		move		#name,r5
		move		#-1,m5
		move		x:(r4),x0  y:(r0),y0                    ;get a,-c
        	mpy		-x0,y0,a  x:(r0)+,x1   y:(r4)+n4,y1     ;-a*c, get d,b
        	macr		-x1,y1,a  x:(r4),x0    y:(r0),y0        ;-b*d, get a,c
 if	@msp(name)==2
        move              a,y:(r5)+                   ; C0 = gain 1
 else
        move              a,x:(r5)+                   ;
 endif
       	do		n0,_gf                             ;do all points
       	mpy		-x0,y0,a  x:(r0)+,x1   y:(r4)+n4,y1     ;-a*c, get d,b
        	macr		-x1,y1,a  x:(r4),x0    y:(r0),y0        ;-b*d, get a,c
	  	asl		a								; Ci<>0 = gain 2
 if	@msp(name)==2
        move              a,y:(r5)+                   
_gf
 else
        move              a,x:(r5)+                   ;save dct
_gf 
 endif
 if @def(name\_busy_b)
	flagclr	name\_busy
 endif

;

	u	a
	u	b
	u	x
	u	y
	u	r0
	u	n0
	u	r1
	u	n1
	u	r2
	u	n2
	u	r4
	u	n4
	u	r5
	u	n5
	u	r6
	u	n6
		endm

;; DSP56300 ASM Code
;; Written by Jean-Marie ORY
;; Free software restricted to academic use


