;; Decimation FIR filter with coefficient generation   y(k) = Sum{x(k-n)*h(n)} n = 0..(N-1)
;; Syntax:  decim_fir2  name,N,flo,fhi,unit,ratio    h(n) = w(n)*{Sin(2*pi*fhi*n)-sin(2*pi*flo*n)}/(pi*n)
;; where: N=number of taps; flo, fhi = bandpass edge freqs; abs= freqs in Hz, rel= freqs in Fs/2 
;; ratio= decimation ratio. Delay line is updated at Fs. Filter output is updated at Fs/ratio 
;; Input: name_in	Output: name  
;; JMO



decim_fir	macro	name,points,flo,fhi,unit,ratio

	if	("unit"=='abs')|("unit"=='Hz')
name\_fu	set	2*flo/actual_fs
name\_fd	set	2*fhi/actual_fs
	else
name\_fu	set	flo
name\_fd	set	fhi
	endif

	var	name\_cntr,ratio
;; Coefficient table 
	section	far	global
	org	y:


name\_inc	set	2.0*pi/@cvf(points-1)
name\_w0	set	0.35875
name\_w1	set	0.48829
name\_w2	set	0.14128
name\_w3	set	0.01168

name\_angle	set	0.0
name\_num	set	(1.-@cvf(points))/2.

name\_coeffs	
	dup	points 
name\_w	set	name\_w0-name\_w1*@cos(name\_angle)+name\_w2*@cos(2.0*name\_angle)-name\_w3*@cos(3.0*name\_angle)
name\_angle	set	name\_angle+name\_inc

	if	name\_num==0.
	dc	name\_w*(name\_fd-name\_fu)
	else
	dc	name\_w*(@Sin(pi*name\_fd*name\_num)-@Sin(pi*name\_fu*name\_num))/(pi*name\_num)
	endif
name\_num	set	name\_num+1.0

	endm
	endsec

;; Ligne  retard suffisante pour ne pas avoir de modif des tats pendant son calcul
name\_lng	equ	points+@cvi(@cvf(points)*actual_fs/fdsp)+20

	bufm	name\_states,x,name\_lng

	org	p:
name\_exe
	section	irq
	flagrst	name\_rdy
	jcnf		name\_new,name\_0
	flagset	name\_rdy
name\_0
	ptr		name\_s,0,name\_states,name\_lng-1
	ldx0		name\_in
	move		x0,x:(r0)+
	ptru		name\_s
	ldr0		name\_cntr
	move		#-1,m0
	nop
	move		(r0)-
	btst		#23,r0
	bcc		name\_1
	move		#ratio,r0
	flagset	name\_dofir
name\_1
	str0		name\_cntr
	u	r0
	u	x
	endsec

	org		p:
	jcnf		name\_dofir,name\_end
	ptr		name\_s,0,name\_states,name\_lng-1
	ptr		name\_c,4,name\_coeffs,points-1,	
	move		(r0)-
	clr	a	x:(r0)-,x0		y:(r4)+,y0
	do		m4,name\_2
	mac		x0,y0,a	x:(r0)-,x0	y:(r4)+,y0	
name\_2
	if		@msp(name)==3
	mac		x0,y0,a		
	else
	macr		x0,y0,a		
	endif
	sta		name
	flagset	name\_new
name\_end
; Used registers for sample execution
	u	x
	u	y
	u	a
	u	r0
	u	r4

	endm
		
;; DSP56300 ASM Code
;; Written by Jean-Marie ORY
;; Free software restricted to academic use


