;; Inverse Discrete Cosine Transform


idct	macro	name

 if !@def(fftbuf)	
	section	longfar		global
		org		L:
n		set		name\_in_size

		buffer	r,n
fftbuf	
		ds		n
		endbuf
fftbuf_size	equ	n
fftbuf_lin	equ	n
fftbuf_col	equ	1
fftbuf_field	equ	3
	endsec
 endif
   
 if !@def(fftcoef)
	section		longfar global
		org		l:
delta	set		2.0*pi/@cvf(name\_size)
angle	set		0.0
		buffer	r,name\_size/2
fftcoef
		dup		name\_size/2
		dc		@lng(@frc(-@cos(angle)),@frc(-@sin(angle)))
angle	set		angle+delta
		endm
		endbuf
	endsec
 endif


;       Generate exponential table after twiddle factor table.
;       This generates the scale factor of EXP(-j*pi*k/(2*N)).
;       The real part is in Y memory and the imaginary part is
;       in X memory.  Note that the negative of the part in Y memory
;       is actually stored so that a -1 (exactly) can be represented
;       with the fractional arithmetic.  This negative sign is
;       compensated when the actual rotation is performed.
;
 if !@def(exptbl)
	section		longfar global
	org	L:
delta	set		pi/@cvf(2*name\_size)
angle	set		0
exptbl
		dup		name\_size
		dc		@lng(@frc(-@sin(angle)),@frc(-@cos(angle)))
angle	set		angle+delta
		endm
		endsec
 endif

;

; Multiply by complex exponential
	org		p:
 if @def(name\_busy_b)
	flagset	name\_busy
 endif

	move		#name\_in,r0		;point to data
	move		#-1,m0			;set linear
	move		#n,n0
	move		#exptbl,r1
	move		m0,m1
	move		#fftbuf,r4
	move		m0,m4
	move		l:(r1)+,x
	do		n0,name\_1
 if	@msp(name\_in)==2
	move		y:(r0)+,y0
 else
	move		x:(r0)+,y0
 endif
	mpy		-x0,y0,a
	mpy		-x1,y0,b	l:(r1)+,x
	move		ab,l:(r4)+		
name\_1


;       Do an IFFT on the new sequence
;
;//        fftr2aa  name\_size,name,name\_coeff        ;do FFT

;; IFFT (adapted from Motorola DSP56300 Library FFTAS.ASM )
         move    #n/2,n0      ;initialize butterflies per group
         move    #1,n2             		;initialize groups per pass
         move    #n/4,n6      ;initialize C pointer offset
         move    #-1,m0            		;initialize A and B address modifiers
         move    m0,m1             ;for linear addressing
         move    m0,m4
         move    m0,m5
         move    #0,m6             ;initialize C address modifier for
                                   ;reverse carry (bit-reversed) addressing
;
; Perform all FFT passes with triple nested DO loop
;
         do      #@cvi(@log(n)/@log(2)+0.5),_end_pass
         move    #fftbuf,r0		;initialize A input pointer
         move    r0,r4				;initialize A output pointer
         move    n0,n1				;initialize pointer offsets
         move    n0,n4
         lua     (r0)+n0,r1			;initialize B input pointer
         move    #fftcoef,r6		;initialize C input pointer
         move    n0,n5
         lua     (r1)-,r5				;initialize B output pointer

         do      n2,_end_grp
         move    x:(r1),x1  y:(r6),y0	;lookup -sine and -cosine values
         move    x:(r5),a   y:(r0),b      ;preload data
         move          x:(r6)+n6,x0  ; update C pointer

         do      n0,_end_bfy
         mac     -x1,y0,b    y:(r1)+,y1    ;Radix 2 DIT ;butterfly kernel
         macr    -x0,y1,b   a,x:(r5)+  y:(r0),a
         subl    b,a        
	   	move    x:(r0),b   b,y:(r4)
         mac     -x1,x0,b   x:(r0)+,a  a,y:(r5)
         macr    y1,y0,b   x:(r1),x1
         subl    b,a        
	   	move    b,x:(r4)+  y:(r0),b
_end_bfy
         move    a,x:(r5)+n5    y:(r1)+n1,y1   ;update A and B pointers
         move    x:(r0)+n0,x1   y:(r4)+n4,y1
_end_grp
         move    n0,b1
         lsr     b   n2,a1			;divide butterflies per group by two
	   nop
         lsl     a   b1,n0			;multiply groups per pass by two
	   nop
         move    a1,n2
_end_pass


idctsort
; Sort IFFT Real data 
	move		#fftbuf,r0
	move		#fftbuf+n-1,r1
	move		#n/2,n0
	move		n0,n1
	move		#0,m0
	move		m0,m1

	move		#name,r4
	move		#-1,m4

	do		n0,name\_2
 if	@msp(name)==2
	move		x:(r0)+n0,x0
	move		x0,y:(r4)+
	move		x:(r1)-n1,x0
	move		x0,y:(r4)+
 else
	move		x:(r0)+n0,x0
	move		x0,x:(r4)+
	move		x:(r1)-n1,x0
	move		x0,x:(r4)+
 endif
name\_2
 if @def(name\_busy_b)
	flagclr	name\_busy
 endif



	u	a
	u	b
	u	x
	u	y
	u	r0
	u	n0
	u	r1
	u	n1
	u	r2
	u	n2
	u	r4
	u	n4
	u	r5
	u	n5
	u	r6
	u	n6
		endm

;; DSP56300 ASM Code
;; Written by Jean-Marie ORY
;; Free software restricted to academic use


