;; Raised cosine minimum bandwidth pulse shaping
;; Syntax: 	rcpulse	name,baud,alpha,len
;; baud = symbol baud rate in Hz
;; alpha = bandwidth rollof length: 0.0 = Rectangle ... 1.0 = Hann window
;; len = pulse length expressed in symbol periods
;; input: name_in = pulse amplitude; 
;; outputs: name = sum of weighted windowed sinc pulses; name_flag = request new input ; 


rcpulse	macro	name,baud,alpha,len 
name\_sps	set	@cvi(actual_fs/baud)		; samples per symbol
name\_fsps	set	@cvf(name\_sps)

		var		name\_ck,0

		bufm		name\_fifo,x,@cvi(len)

		section	data	global
name\_pos	set		-@cvf(name\_sps*len)/2.+0.5
		org		y:
		buffer		m,@cvi(name\_sps*len)
name\_sinc
		dup		@cvi(name\_sps*len)
name\_snc	set		@sin(pi*name\_pos/name\_fsps)/(pi*name\_pos/name\_fsps)
name\_win	set		@cos((pi*name\_pos/name\_fsps)*alpha)/(1-(4*alpha*alpha*(name\_pos/name\_fsps)*(name\_pos/name\_fsps)))
		dc		name\_snc*name\_win
name\_pos	set		name\_pos+1.0
		endm
		endbuf
		endsec

		org	p:
name\_exe
		ptr		name\_ptx,0,name\_fifo,len-1
		ptr		name\_pty,4,name\_sinc,((name\_sps*len)-1),name\_sps
 if @def(name\_d_b)
		flagrst	name\_d
 endif
 if @def(name\_u_b)
		flagrst	name\_u
 endif

		timerp		name\_t,name\_sps/2
		jcnf		name\_t,name\_1
		move		#0.999999,a
		flagtog	name\_th
		clr		a	ifcc
		sta		name\_ck
		bcs		<name\_0
 if @def(name\_d_b)
		flagset	name\_d
 endif
		bra		<name\_1
name\_0	lda		name\_in
		move		(r0)-
		move		a,x:(r0)
		move		#name\_sinc,r4
		ptru		name\_ptx
 if @def(name\_d_b)
		flagset	name\_u
 endif
name\_1
		clr		a		x:(r0)+,x0	y:(r4)+n4,y0
		rep		m0
		mac		x0,y0,a	x:(r0)+,x0	y:(r4)+n4,y0
		macr		x0,y0,a	(r4)+
		ptru		name\_pty
		sta		name

		u	a
		u	x
		u	y
		u	r0
		u	r4
		u	n4
		endm
;; DSP56300 ASM Code
;; Written by Jean-Marie ORY
;; Free software restricted to academic use


