;; RT Spectrum analyser with oscilloscope display  
;; Syntax:	specan	size,full|half,pow|amp|db,<window>,dc|ac
;; fs= sample fr., size= power of 2 (max 1024), displ mode= pow, amp or dB, r/c = real/complex input,  
;; size=data size display= full/half, win= do windowing, ac: highpass input;
;; Input = specan_in (real or complex)   Output= da1: scope scan with sync pulse

specan	macro	size,display,mode,wino,decim
	var	specan_acqptr,specan_acq
	var	specan_visptr,specan_visu
 if decim>1
	var	specan_cntr,1
 endif	

	if	@def(specan_in_im)
	bufm		specan_acq,l,size
	else
	bufm		specan_acq,x,size
	endif

 
	bufr		specan_fftbuf,l,size
	if		"display"!='full'
	bufm		specan_pspec,l,size/2
	bufm		specan_visu,y,size/2   ;; +4
specan_endvisu set specan_visu+size/2
	else
	bufm		specan_pspec,l,size
	bufm		specan_visu,y,size+4
specan_endvisu set specan_visu+size
	endif
 if	"wino"=='Rectangle'
specan_som	set	size
 endif

 if	"wino"!='Rectangle'
	section	data	global

  if "wino"=='Triangle'
specan_som	set	0.0
specan_wmax	set	0.0
specan_n	set	0
	dup	size
w	set	2.*(0.5*@cvf(size)-@abs(specan_n-0.5*@cvf(size-1)))/@cvf(size)
 if	w>specan_wmax
specan_wmax	set	w
 endif
specan_n	set	specan_n+1
	endm
specan_n	set	0
	org	y:	
specan_win
	dup	size
w	set	2.*(0.5*@cvf(size)-@abs(specan_n-0.5*@cvf(size-1)))/@cvf(size)
	dc	w/specan_wmax
specan_som	set	specan_som+(w/specan_wmax)
specan_n	set	specan_n+1
	endm
  endif

  if "wino"=='Hann'
specan_som	set	0.0
specan_wmax	set	0.0
specan_n	set	0
	dup	size
w	set	0.5*(1.0-@cos(2.*pi*@cvf(specan_n)/@cvf(size-1)))
 if	w>specan_wmax
specan_wmax	set	w
 endif
specan_n	set	specan_n+1
	endm
	org	y:	
specan_win
specan_n	set	0
	dup	size
w	set	0.5*(1.0-@cos(2.*pi*@cvf(specan_n)/@cvf(size-1)))
	dc	w/specan_wmax
specan_som	set	specan_som+(w/specan_wmax)
specan_n	set	specan_n+1
	endm

  endif

  if "wino"=='Hamming'
specan_som	set	0.0
specan_wmax	set	0.0
specan_n	set	0
	dup	size
w	set	0.54-0.46*@cos(2.*pi*@cvf(specan_n)/@cvf(size-1))
 if	w>specan_wmax
specan_wmax	set	w
 endif
specan_n	set	specan_n+1
	endm
	org	y:	
specan_win
specan_n	set	0
	dup	size
w	set	0.54-0.46*@cos(2.*pi*@cvf(specan_n)/@cvf(size-1))
	dc	w/specan_wmax
specan_som	set	specan_som+(w/specan_wmax)
specan_n	set	specan_n+1
	endm

  endif


  if "wino"=='Gauss'
specan_sigma	set	0.2
specan_som	set	0.0
specan_wmax	set	0.0
specan_n	set	0
	dup	size
specan_p set	@cvf(specan_n)-@cvf(size-1)/2.
specan_q	set	specan_sigma*@cvf(size-1)/2.
w	set	@xpn(-0.5*specan_p*specan_p/(specan_q*specan_q))
 if	w>specan_wmax
specan_wmax	set	w
 endif
specan_n	set	specan_n+1
	endm

	org	y:
specan_win
specan_n	set	0
	dup	size
specan_p set	@cvf(specan_n)-@cvf(size-1)/2.
specan_q	set	specan_sigma*@cvf(size-1)/2.
w	set	@xpn(-0.5*specan_p*specan_p/(specan_q*specan_q))
	dc	w/specan_wmax
specan_som	set	specan_som+(w/specan_wmax)
specan_n	set	specan_n+1
	endm
  endif



  if "wino"=='Blackman_Harris'
specan_inc1	set	2.0*pi/@cvf(size-1)
specan_w0	set	0.35875
specan_w1	set	0.48829
specan_w2	set	0.14128
specan_w3	set	0.01168

specan_angle	set	0.0
specan_som	set	0.0
specan_wmax	set	0.0
	dup	size
w	set	specan_w0-specan_w1*@cos(specan_angle)+specan_w2*@cos(2.0*specan_angle)-specan_w3*@cos(3.0*specan_angle)
 if	w>specan_wmax
specan_wmax	set	w
 endif
specan_angle	set	specan_angle+specan_inc1
	endm
	org	y:
specan_win
specan_angle	set	0.0
	dup	size
w	set	(specan_w0-specan_w1*@cos(specan_angle)+specan_w2*@cos(2.0*specan_angle)-specan_w3*@cos(3.0*specan_angle))
	dc	w/specan_wmax
specan_som	set	specan_som+(w/specan_wmax)
specan_angle	set	specan_angle+specan_inc1
	endm
  endif


  if "wino"=='Nuttall'
specan_inc1	set	2.0*pi/@cvf(size-1)
specan_w0	set	0.3635819
specan_w1	set	0.4891775
specan_w2	set	0.1365995
specan_w3	set	0.0106411

specan_angle	set	0.0
specan_som	set	0.0
specan_wmax	set	0.0
	dup	size
w	set	specan_w0-specan_w1*@cos(specan_angle)+specan_w2*@cos(2.0*specan_angle)-specan_w3*@cos(3.0*specan_angle)	
 if	w>specan_wmax
specan_wmax	set	w
 endif
specan_angle	set	specan_angle+specan_inc1
	endm
	org	y:
specan_win
specan_angle	set	0.0
	dup	size
w	set	(specan_w0-specan_w1*@cos(specan_angle)+specan_w2*@cos(2.0*specan_angle)-specan_w3*@cos(3.0*specan_angle))
	dc	w/specan_wmax
specan_som	set	specan_som+(w/specan_wmax)
specan_angle	set	specan_angle+specan_inc1
	endm
  endif

  if "wino"=='Flat_Top'
specan_inc1	set	2.0*pi/@cvf(size-1)
specan_w0	set	1.0
specan_w1	set	1.93
specan_w2	set	1.29
specan_w3	set	0.388
specan_w4	set	0.032


specan_angle	set	0.0
specan_som	set	0.0
specan_wmax	set	0.0
	dup	size
w	set	specan_w0-specan_w1*@cos(specan_angle)+specan_w2*@cos(2.0*specan_angle)-specan_w3*@cos(3.0*specan_angle)+specan_w4*@cos(4.0*specan_angle)
 if	w>specan_wmax
specan_wmax	set	w
 endif
specan_angle	set	specan_angle+specan_inc1
	endm
	org	y:
specan_win
specan_angle	set	0.0
	dup	size
w	set	(specan_w0-specan_w1*@cos(specan_angle)+specan_w2*@cos(2.0*specan_angle)-specan_w3*@cos(3.0*specan_angle)+specan_w4*@cos(4.0*specan_angle))
	dc	w/specan_wmax
specan_som	set	specan_som+(w/specan_wmax)
specan_angle	set	specan_angle+specan_inc1
	endm
  endif
	endsec
 endif

;Sine-Cosine Table

	section	longfar	global	

specan_angle	set	0.0
specan_incr	set	2*pi/@cvf(size)

	org	l:
	buffer	r,size/2
specan_twid
	dup	size/2
	dc	@lng(@frc(-@cos(specan_angle)),@frc(@sin(specan_angle)))
specan_angle	set	specan_angle+specan_incr
	endm

	endbuf
		
	endsec



	section	inits	
	move	#-1.0,a
	if	"display"!='full'
	move	a,y:specan_visu+size/2
	move	a,y:specan_visu+size/2+1
	move	a,y:specan_visu+size/2+2
	move	a,y:specan_visu+size/2+3
	else
	move	a,y:specan_visu+size
	move	a,y:specan_visu+size+1
	move	a,y:specan_visu+size+2
	move	a,y:specan_visu+size+3
	endif
	endsec

specan_loop
;	INPUT WINDOWING

	move	x:specan_acqptr,r1
	move	#size-1,m1
	move	#specan_fftbuf,r5
	move	#-1,m5
  
	if	"wino"!='Rectangle'
	move	#specan_win,r4
	move	#-1,m4
	endif

	move	#size,n1
 if	!@def(specan_in_im)
;;  if		"acdc"=='dc'
	move	#0,x0
	do	n1,specan1

   if	"wino"!='Rectangle'
	move		x:(r1)+,x1	y:(r4)+,y0
	mpyr		x1,y0,a		x0,y:(r5)
	move		a,x:(r5)+
   else
	move		x:(r1)+,x1
	move		x,l:(r5)+
   endif
specan1
 else
	do	n1,specan1
	move		l:(r1)+,x
 if	"wino"!='Rectangle'
	move		y:(r4)+,y0
	mpyr	x0,y0,a
	mpyr	x1,y0,b
	move	ab,l:(r5)+
 else
	move	x,l:(r5)+
 endif

specan1
 endif

	dbgdisa


;	FFT algorithm	(source: Motorola library)

	move		#size/2,n0      ;initialize butterflies per group
	move		#@cvi(@log(size)/@log(2)+0.5),n1
	move		#size/4,n6      ;initialize C pointer offset
	move		#specan_fftbuf,r2
	move		#specan_twid,r3
	jsr		fft

;	READ and AVERAGE POWER SPECTRUM

	move		#specan_fftbuf,r1
	move		#size/2,n1
	move		#0,m1
	move		#specan_pspec,r2	
	move		#-1,m2
	move		l:(r1)+n1,x

	move		#>@cvf(size)/(specan_som*8.0),y0 ; compensation du gain de fentre

	if		"display"!='full'
	do		#size/2,specan_3
	else
	do		#size,specan_3
	endif

 if	"wino"=='Rectangle'
	mpy		x0,x0,a		x1,x0
	mac		x0,x0,a		l:(r2),b	
 else
	mpy		x1,y0,a
	asl		#3,a,a
	mpy		x0,y0,b		a,x0
	asl		#3,b,b
	mpy		x0,x0,a		b,x0
	mac		x0,x0,a		l:(r2),b
 endif
	sub		b,a			l:(r1)+n1,x
	asl		#6,b,b	;//
	add		b,a
	asr		#6,a,a	;//
	move		a,l:(r2)+
specan_3
	move		m2,m1

; MAKE SCOPE OUTPUT

	move		#specan_visu,r4
	move		m1,m4
	if		"display"!='full'
	move		#specan_pspec,r1
	move		#-1,m1
	do		#size/2,specan_4
	else
	move		#specan_pspec+size/2,r1
	move		#size-1,m1
	do		#size,specan_4
	endif

	move		l:(r1)+,a
	if		"mode"=='Amp'
	jsr		sqrt12
	endif

	if		"mode"=='dB'
	jsr		decibel
	endif

	move		a,y:(r4)+
specan_4

	if	!@def(fft)

	section	routines
	org	p: 
	global fft
	
fft	
         move    #1,n2             		;initialize groups per pass
         move    #-1,m0            		;initialize A and B address modifiers
         move    m0,m1             ;for linear addressing
         move    m0,m4
         move    m0,m5
         move    #0,m6             ;initialize C address modifier for
                                   ;reverse carry (bit-reversed) addressing
         andi    #$7F,ccr          ;clear scaling bit
         ori     #$4,mr            ;set scaling down mode
;
; Perform all FFT passes with triple nested DO loop
;
         do      n1,_end_pass
         move    r2,r0			;initialize A input pointer
         move    r0,r4				;initialize A output pointer
         move    n0,n1				;initialize pointer offsets
         move    n0,n4
         lua     (r0)+n0,r1			;initialize B input pointer
         move    r3,r6		;initialize C input pointer
         move    n0,n5
         lua     (r1)-,r5				;initialize B output pointer

         do      n2,_end_grp
         move    x:(r1),x1  y:(r6),y0	;lookup -sine and -cosine values
         move    x:(r5),a   y:(r0),b      ;preload data
         asl     a          x:(r6)+n6,x0  ;since scaling mode on a=2*a,update C pointer
;         move          x:(r6)+n6,x0  ;since scaling mode on a=2*a,update C pointer

         do      n0,_end_bfy
         mac     x1,y0,b    y:(r1)+,y1    ;Radix 2 DIT ;butterfly kernel
         macr    -x0,y1,b   a,x:(r5)+  y:(r0),a
         subl    b,a        
	   	move    x:(r0),b   b,y:(r4)
         mac     -x1,x0,b   x:(r0)+,a  a,y:(r5)
         macr    -y1,y0,b   x:(r1),x1
         subl    b,a        
	   	move    b,x:(r4)+  y:(r0),b
_end_bfy
         move    a,x:(r5)+n5    y:(r1)+n1,y1   ;update A and B pointers
         move    x:(r0)+n0,x1   y:(r4)+n4,y1
_end_grp
         move    n0,b1
         lsr     b   n2,a1			;divide butterflies per group by two
	   nop
         lsl     a   b1,n0			;multiply groups per pass by two
	   nop
         move    a1,n2
_end_pass
         andi    #$7f,ccr			;clear scaling bit
         andi    #$f3,mr			;no scaling
;		nop
		rts

	endsec
	endif


	if	!@def(sqrt12)
	section	routines
	global	sqrt12
	org	p:	
sqrt12	move	a,y1
	move	a0,y0
	clr	a	#0.5,x0
	move	x0,x1
	do	#12,_sqrt1
	mpy	-x0,x0,b
	add	y,b
	tge	x0,a
	tfr	x1,b
	asr	b
	nop 
	add	a,b	b,x1
	nop
	move	b,x0
_sqrt1
	rts
	endsec
	endif


	if	!@def(decibel)
	section	routines
	global	decibel
	
;; dB = 10 log(2).lg2(x)
;; dB/200 = 10 x 0.30103 x 0.32 lg2(x)/64  = 0.96330 . lg2(x) / 64
;; dB/140 = 10 x 0.30103 x 0.22857 log2(x)/32 = 2 x (0.68807 lg2(x) /64)
	org	p:
decibel	tst	a
	bgt	<_db0
	move	#-1.0,a
	rts
_db0	bsr	flog2	; a = log2(x) [-47 ... +8[
	asr	#6,a,a	; div /64 pour que |A| < 1

	
	move	a,x0
;;	mpyi	#0.96330,x0,a	; 0dB --> a = 0  -200dB --> a = -1
	mpyi	#0.68807,x0,a	
	asl	a			; 0dB --> a = 0  -140dB --> a = -1
;;	asl	a		; 0..-100 dB	
;;	add	#0.5,a	
;;	asl	a		; 0dB --> a = 1  -100dB -> a=0   -200dB -> a=-1
	rts
	endsec
	endif

	if	!@def(flog2)

	section	data	global
	org	y:	
lg2coef dc	-0.236856,0.631994,-0.875888,0.800231,-0.406756,0.087275
;NB: coeffs de x^0 a x^5    (a multiplier par 16)
	endsec

	section	routines
	global	flog2

	org	p:
flog2	move	#lg2coef,r0
	move	#-1,m0
	clb	a,b
	normf	b1,a
	move	a,x0
	move	b1,n0

	move	y:(r0)+,a			; a = k0
	mpyr	x0,x0,b		y:(r0)+,y1	; b = x^2	y1 = k1	

	mac	x0,y1,a b,x1	y:(r0)+,y1 	; a = a+k1.x  y1 = k2 x1=x^2 
	mpyr	x0,x1,b		b,y0		; b = x^3	y0 = x^2

	mac	y0,y1,a		y:(r0)+,y1	; a = a+k2.x^2  y1 = k3
	mpyr	y0,y0,b		b,x1		; b = x^4	x1 = x^3

	mac	x1,y1,a		y:(r0)+,y1	; a = a+k3.x^3  y1 = k4
	mpyr	x1,y0,b		b,x0		; b = x^5	x0 = x^4
	
	mac	x0,y1,a	b,x0	y:(r0)+,y1	; a = a+k4.x^4  y1 = k5	x0 = x^5
	mac	x0,y1,a		#0,b		; a = a+k5.x^5
	asl	#4,a,a				; a = 16a
	move	n0,b2				; n0 = fact. puissance de 2
	asr	b
	add	b,a	; a = log2(a initial) = -47 ... +8
	rts
	endsec
	endif	; not def flog2


	section	irq
	org	p:
critic	gset		1
 if	(decim>1)
	lda		specan_cntr
	sub		#1,a
	sta		specan_cntr
	bgt		<specan_5
	move		#decim,a1
	sta1		specan_cntr
 endif

	ldr0		specan_acqptr
	move		#size-1,m0

	if		@def(specan_in_im)
	lda		specan_in_re
	move		a,x:(r0)
	lda		specan_in_im
	move		a,y:(r0)+
	else
	lda		specan_in
	move		a,x:(r0)+
	endif
	
	str0		specan_acqptr
 
specan_5
	if		"display"!='full'
	move		#specan_visu+size/2-1,m0
	else
	move		#specan_visu+size-1,m0 
	endif
	jnf		scope_sync,specan_5a
	move		#specan_visu,r0
	nop 
	move		y:(r0)+,a
	sta		specan
	str0		specan_visptr
	bra		<specan_6
specan_5a		
	ldr0		specan_visptr
;	if		"display"!='full'
;	move		#size/2-1,m0
;	else
;	move		#size-1,m0
;	endif

	move		r0,a
	cmp		#>specan_visu,a
	bne		specan_5b
	flagset	scope_sync
specan_5b
	move		y:(r0)+,a
	sta		specan
	str0		specan_visptr
;	move		r0,a
;	cmp		#>specan_visu,a
;	bne		specan_6
;	flagset	scope_sync
specan_6
	u	a
	u	r0
critic gset	0
	endsec
	
	endm

;; DSP56300 ASM Code
;; Written by Jean-Marie ORY
;; Free software restricted to academic use


