	section routines
	global floatlib
	global	fabs_x,fabs_a,fadd_xy,fadd_xa,fceil_x,fceil_a,fcmp_xy,fcmp_xa,fdiv_xy,fdiv_xa
	global	fix_x,fix_a,float_x,float_a,floor_x,floor_a,frac_x,frac_a,fmac_xya,fmac_mxya
	global	fmpy_xy,fmpy_xa,fneg_x,fneg_a,fscal_xy,fscal_xa,fsqrt_x,fsqrt_a,fsub_xy,fsub_xa

floatlib

; FPINIT
	section data global
        org     y:
fp_temp ds      4       		;temporary storage locations
fp_23   dc      23      		;exponent delta limit
fp_ebias dc     $001fff 		;fixed point exponent bias
fp_emax dc      $003fff 		;maximum exponent
fp_m1   dc      $ffffff
	endsec

        org p:
fpinit  move    y:fp_m1,m0   ;initialize linear address modifier
        and     #$f3,mr         	;reset scaling modes (S1,S0)
        and     #$bf,ccr        	;reset overflow flag (L bit)
        rts


; FPABS
fabs_x  tfr     x0,b    x1,a            ;get ex, mx
fabs_a  tst     a                       ;check for negative ma
        jmi     fneg_a                  ;negate if negative
        rts

; FPADD
fadd_xy tfr     y0,b    y1,a            ;get ey, my
fadd_xa cmp     x0,b    y:fp_23,y0       ;compare delta = er - ea,
                                                ; get delta limit
        jge     _dpos                   ;jump if er >= ea
;
; er < ea
;
_dneg   tfr     x0,b    b1,x0           ;swap ea with er
	  tfr     x1,a    a1,x1           ;swap ma with mr
;
; er >= ea
;
_dpos   sub     x0,b    b1,y1           ;calculate delta, save er'
        cmp     y0,b               	    ;check delta limit
        jgt     done1                   ;jump if delta > 23
;
; normalize and round result - assumes a=mr', r0=er' and CCR reflects mr' value.
;
addm	  tfr	x1,b	b1,y0    
	  asr	y0,b,b				;denormalize by delta
	  add	b,a					;add mantissa
norm	  clb	a,b
	  normf	b1,a				;normalization
round   rnd     a 	b,y0   		;round to 24 bit mr
	  clb	a,b	
	  normf	b1,a				;correct mr overflow due to round
	  add		y1,b				
	  add		y0,b				;get final er
	  nop
;
; detect and correct exceptions - assumes a=mr and r0=er
;
check   jset    #15,b1,under      		;jump if exponent underflow
        jset    #14,b1,limit      		;jump if exponent overflow
check1  tst     a      				;check mr
        teq     a,b                     	;if mr=0, correct to fp zero
        rts
under   or      #$40,ccr                	;set L=1 for exponent underflow
zero    clr     a       #0,b            	;correct to fp zero
        rts
limit   asl     a       y:fp_emax,y1     ;correct to maximum fp value
	  nop
done1   tfr     y1,b    a,a             	;get mr and er, set L bit if mr limited
done    rts

; FPCEIL
fceil_x tfr     	x0,b    x1,a            ;get mx,ex
	  nop
fceil_a
        neg     	a      	b,y1        ;reverse number line,save ea
	  clb		a,b
	  normf	b1,a	
	  add		y1,b				;normalize the input for the floor operation
        jsr     	floor_a                 ;do floor operation
        neg     	a       b,y1            ;reverse number line,save ea
	  clb		a,b
	  normf	b1,a	
	  add		y1,b				;normalize the output 
        rts

; FPCMP
fcmp_xy tfr     y0,b    y1,a            ;get ey, my
	  nop
fcmp_xa eor     x1,a    a,y0            ;compare signs, save ma
        jmi     _mant                   ;jump if different signs
;
; Same signs
;
_sign   eor     x1,a                    ;check sign of ma
        jpl     _sign1                  ;jump if ma and mx positive

        tfr     x0,b    b,x0            ;if minus, exchange ea and ex
_sign1  cmp     x0,b                    ;compare exponents
        jne     done                    ;jump if different exponents
;
; Different signs, same exponents or different signs
;
_mant   tfr     y0,a                    ;restore ma
        cmp     x1,a                    ;compare mantissa
        rts

;FPDIV
fdiv_xy tfr     y0,b    y1,a            ;get ey, my
fdiv_xa sub     x0,b    y:fp_ebias,x0 ;calculate er' = ea - ex,get ebias
        add     x0,b                    ;add ebias to er'
	  nop
        tfr     x1,b    b,r0            ;get mx, save er'
        tst     b                       ;check for divisor mx = 0
        jne     _div1                   ;jump if divisor not zero
        tst     a                       ;check for dividend ma = 0
        jne     limit                   ;jump if dividend not zero
        or      #$40,ccr                ;set L=1 for divide by zero error
_div1   asr     a       a,b             ;force fractional quotient, copy ma
        jeq     done                    ;jump if dividend ma = 0 (underflow)
        abs     a       (r0)+           ;make dividend positive, adjust er'
        eor     x1,b    r0,y1           ;calculate quotient sign,get er'
        and     #$fe,ccr                ;clear carry (quotient sign bit)
        rep     #24                     ;repeat for 24 bit quotient
        div     x1,a                    ;generate one quotient bit
        jpl     _qpos                   ;jump if positive quotient
        neg     a                       ;negate mr' for negative quotient
	  nop
_qpos   move    a0,a                    ;get quotient mr'
        jmp     norm                    ;normalize and check for errors

;FPFIX
fix_x   tfr     x0,b    x1,a            ;get mx,ex
fix_a   tst     a       y:fp_ebias,x0    ;check ma, get bias
        jeq     done                    ;jump if ma = 0
        sub     x0,b    #>8,x0          ;remove ea bias, get shift limit
        jlt     _eneg                   ;jump if exponent < 0
        jeq     done                    ;jump if exponent = 0
;
; exponent > 0
;
        cmp     x0,b                    ;check left shift limit
        jgt     _ehigh                  ;jump if exponent > 8
	  asl	    b1,a,a			    ;shift left b bits
        rts
_ehigh  or      #$40,ccr                ;set L=1 for conversion overflow
        tst     a                       ;check sign of ma
        jpl     _fix1                   ;jump if ma positive
        clr     a                       ;limit to maximum negative 56 bit value
        move    #$80,a2                 ;set extension word
        rts
_fix1   clr     a                       ;limit to maximum positive 56 bit value
        not     a    		          ;set most significant word
	  nop
        move    a1,a0                   ;set least significant word
        move    #$7f,a2                 ;set extension word
        rts
;
; exponent < 0
;
_eneg	  abs	    b		#>48,x0	    ;fix exponent, get right shift limit
	  cmp	    x0,b			    ;check right shift limit
	  jge	    zero			    ;jump if exponent >= 48
	  asr	    b1,a,a			    ;shift right b bits
	  rts	

; FPFLOAT
float_x tfr     	x1,a                    ;get data
float_a tst     	a       y:fp_ebias,y1		;check data, initialize er
        jnr     	float1                  ;jump if data normalized
	  clb		a,b
	  normf	b1,a			    	;normalization
	  add		y1,b			
	  nop
	  move	b,y1			    	;get new er
float1  rnd     	a 	   		    	;round to 24 bit mr
	  clb		a,b	
	  normf	b1,a	
	  add		y1,b			    	;correct mr overflow due to round
        jmp     	check1                  ;go check exponent range

;FPFLOOR
floor_x	tfr	x0,b	x1,a		;get mx,ex
floor_a
	tst	a	#$1fff+23,x0	;check a, max exponent w/o fraction
	jeq	_done			;if a=0, result=0
	cmp	x0,b	y:fp_ebias,x0	;check for max input
	jge	_done			;if max, result=input
	cmp	x0,b	b,x1		;check for fraction, save exponent
	jgt	_inrange		;jump if not less than one
	and	#$800000,a		;if 0<input<1 --> output=0,if -1<=input<0 --> output=-1
	tst	a	#0,b		
	tmi	x0,b			;if 0<input<1 --> er=$000000,if -1<=input<0 --> er=$001fff
	rts
_inrange
	sub	x0,b			;subtract bias,get real exponent
	neg	b
	add	#23,b			;23-real exponent
	lsr	b1,a
	lsl	b1,a			;clear the less significant (23-real exponent) bits
	move	x1,b			;restore exponent
_done
	rts

; FPFRAC
frac_x
        tfr     x0,b    x1,a            ;get mx,ex
	  nop
frac_a  
        move    b1,y0                   ;copy input
        move    a1,y1
        jsr     floor_a                 ;get floor of input
        move    b1,x0                   ;put floor in X
        move    a1,x1
        jmp     fsub_xy                 ;R=x-floor(x)

; FPMAC
fmac_xya move   a,y:fp_temp+1    ;save ma
        move    b,y:fp_temp+2    ;save ea
        jsr     fmpy_xy                 ;multiply x*y first
        move    y:fp_temp+1,x1   ;restore ma
        move    y:fp_temp+2,x0   ;restore ea
        jmp     fadd_xa                 ;add product to a
;
fmac_mxya move  a,y:fp_temp+1    ;save ma
        move    b,y:fp_temp+2    ;save ea
        jsr     fmpy_xy                 ;multiply x*y first
        move    a,x1                    ;save product mantissa
        move    b,x0                    ;save product exponent
        move    y:fp_temp+1,a    ;restore ma
        move    y:fp_temp+2,b    ;restore ea
        jmp     fsub_xa                 ;subtract product from a

;FPMPY

fmpy_xy tfr     y0,b    y1,a            	;get ey, my
fmpy_xa add     x0,b    y:fp_ebias,x0    ;er' = ex + ea,get ebias
        sub     x0,b    a,y1            	;subtract ebias from er',get ma
	  nop
        mpy     x1,y1,a         b,y1    	;mr = mx * ma, get er'
        jmp     norm                   	;normalize and check result

; FPNEG
fneg_x  tfr     x0,b    x1,a            ;get ex, mx
	  nop
fneg_a  neg     a       b,y1            ;negate ma, save ea
        jmp     norm                    ;normalize ma and check ea

; FPSCALE
fscal_xy        tfr     y0,b    y1,a    ;get ey, my
fscal_xa        add     x0,b            ;add scale factor to ea
        	  jmp     check           ;check for exponent errors

; FPSQRT
fsqrt_x tfr     x0,b    x1,a            		;get mx,ex
fsqrt_a tst     a       y:fp_ebias,x0    ;check for negative input
        jmi     under                   		;jump if negative, set L bit
        jeq     done                    		;jump if zero
        sub     x0,b                    		;remove ebias from ea
        asr     b       #<$40,y0        		;er' = ea / 2, get first guess
        jclr    #23,b0,_sqrt1                  	;jump if er' even
        asr     a       y:fp_m1,x1       ;divide ma by 2, get -1
        sub     x1,b                    		;increment er' by 1
_sqrt1  add     x0,b    y0,x0           		;add ebias to er', save first temp
	  move    a,x1			    		;save ma
        clr     b       b,r0            		;clear root, save er'
        do      #23,_sqrt2              		;square all 23 bits
        mac     -x0,x0,a            		;square temp
        tge     x0,b                    		;update root if correct guess
	  mpy     y0,#1,a					;try next bit
	  nop
        add     b,a     a,y0            		;form root guess, save guess bit
	  nop
        tfr     x1,a    a,x0            		;get ma, save temp
_sqrt2
        tfr     b,a     r0,b            		;get root mr, er
        rts

; FPSUB
fsub_xy tfr     y0,b    y1,a            	;get ey, my
	  nop  	
fsub_xa sub     x0,b    b1,y1           	;compare delta = ea - ex, save ea
        jge     _dspos                   	;jump if ea >= ex
;
; ea < ex
;
_dsneg   tfr     x1,a    a1,x1           	;swap ma with mx
        neg     a       x0,y1           	;negate mx, save ex
        abs     b       y:fp_23,y0       ;negate er', get delta limit
        cmp     y0,b              		;check delta limit
        jle     addm                    	;jump if delta =< 23
        jmp     norm                   	;normalize result
;
; ea >= ex
;
_dspos   move            y:fp_23,y0       ;get delta limit
        cmp     y0,b               		;check delta limit
        jgt     done1                   	;jump if delta > 23
	  tfr	    x1,b	b1,y0    
	  asr	    y0,b,b				;denormalize
	  sub	    b,a				;subtract mantissa
        jmp     norm                    	;normalize result
	
	u	a
	u	b
	u	x
	u	y
	u	r0

	endsec
;; DSP56300 ASM Code
;; Written by Jean-Marie ORY
;; Free software restricted to academic use
;; Last update 21/12/2008

